
package typo.parse;

import java.util.*;
import typo.term.*;

/** A simple symbol table, using a Hashtable, and Vectors
 * for scoping. */
public class SymbolTable {

  /** The Hashtable of Vectors containing the symbols
   * in question.  The variable of each name currently
   * in scope is at the <b>end</b> of the Hashtable. */
  private Hashtable h = new Hashtable();

  /** Constructor. */
  public SymbolTable() {
  }

  /** Declare a variable, creating a new Var object with
   * the given name.
   * @param name  the name of the variable */
  public void declare(String name) {
    Vector v = (Vector) h.get(name);
    if (v == null) {
      v = new Vector();
      h.put(name, v);
    }
    v.addElement(new Var(name));
  }

  /** Declare a variable, which already is a Var object.
   * (this may go away...)
   * @param x  the Var object in question */
  public void declare(Var x) {
    Vector v = (Vector) h.get(x);
    if (v == null) {
      v = new Vector();
      h.put(x.name, v);
    }
    v.addElement(x);
  }

  /** "Undeclare" a variable, removing it from scope.  Should
   * be called when we're done parsing some construct.
   * @param name  the name of the variable to undeclare. */
  public void undeclare(String name) {
    Vector v = (Vector) h.get(name);
    v.removeElementAt( v.size() - 1 );
  }

  /** Look up a variable.
   * @param name  the name of the variable to look up
   * @return  that variable, or null if it wasn't found. */
  public Var lookup(String name) {
    Vector v = (Vector) h.get(name);
    return (Var) v.elementAt( v.size() - 1 );
  }
}

