package typo.random;

import java.util.*;

import math.util.*;
import typo.term.*;

/** Computes the distribution of values a (stochastic) term
 * might have. */
class Dist {

  /** The distribution of terms. */
  public Vector dist = new Vector();

  /** Constructor. */
  public Dist(Term x) {
    dist.addElement(new DistEntry(0, x));
  }

  /** Reduce all possibilities once.
   * returns  true iff any term wasn't in normal form */
  public boolean expand() {
    Vector newDist = new Vector();
    boolean changed = false;

    // loop through all the possibilities
    for(Enumeration e = dist.elements(); e.hasMoreElements(); ) {
      DistEntry d = (DistEntry) e.nextElement();
      Vector v = d.x.dist();

      for(Enumeration e1 = v.elements(); e1.hasMoreElements(); ) {
        DistEntry d1 = (DistEntry) e1.nextElement();

        // if something's not in normal form, then we're not done
        if (! d1.x.headReduceOnce().alphaEquals(d1.x))
          changed = true;
        newDist.addElement( new DistEntry(d.p + d1.p, d1.x) );
      }
    }

    dist = newDist;
    return changed;
  }

  /** Simplify this distribution. */
  public void simplify() {
    Vector x = new Vector();     // the Terms seen so far
    Vector a = new Vector();     // the LogAdder objects

    // this runs in quadratic time, alas.
    for(int i=0; i<dist.size(); i++) {
      DistEntry d = (DistEntry) dist.elementAt(i);
      int termIndex = -1;

      // look for a's term amongst what we've seen so far
      for(int j=0; j<x.size(); j++) {

        if (d.x.alphaEquals( (Term) x.elementAt(j) )) {
          termIndex = j;
          break;
        }
      }

      if (termIndex == -1) {   // we haven't seen this term so far
        x.addElement( d.x );
        LogAdder la = new LogAdder();
        la.add( d.p );
        a.addElement( la );
      }
      else {    // we have seen it, so add to its probability
        LogAdder la = (LogAdder) a.elementAt( termIndex );
        la.add( d.p );
      }
    }

    // now, create a new distribution
    Vector v = new Vector();
    for(int i=0; i<x.size(); i++) {
      Term t = (Term) x.elementAt(i);
      LogAdder la = (LogAdder) a.elementAt(i);
      v.addElement( new DistEntry(la.logSum(), t));
    }

    // update dist
    dist = v;
  }

  /** Convert this to a String. */
  public String toString() {
    return dist.toString();
  }
}

