
package typo.term;

import java.util.*;

/** A lambda abstraction. */
public class Abstr implements Term {

  /** The bound variable.  When the Abstraction is copied, or
   * substituted into, this is copied as well. */
  public Var x;

  /** The body of the abstraction. */
  public Term s;

  /** Constructor. */
  public Abstr(Var x, Term s) {
    this.x = x; this.s = s;
  }

  /** Substitute into this term. */
  public Term substitute(Var y, Term t) {
    // XXX this wastes memory
    return new Abstr(x, s.substitute(y, t));
  }

  /** To reduce this, reduce inside the term's body.
   * Or reduce inside the type. */
  public Term headReduceOnce() {

    // eta reduction
    // (not used for the stochastic language)
/*
    if (s instanceof Appl) {
      Appl a = (Appl) s;
      if (a.rand.equals(x))
        return a.rator;
    }
*/

    return new Abstr(x, s.headReduceOnce());
  }

  public boolean alphaEquals(Term t) {
    if (t instanceof Abstr) {   // if x is also an abstraction,
      Abstr a1 = (Abstr) t;

      // rename the other var. to x, and compare
      return s.alphaEquals( a1.s.substitute(a1.x, x));
    }

    return false;    // they're not both lambda abstractions
  }

  public Vector dist() {
    Vector v = new Vector();

    // The distribution of this term is the distribution over
    // s, put into abstractions
    Vector sDist = s.dist();
    for(Enumeration e = sDist.elements(); e.hasMoreElements(); ) {
      DistEntry d = (DistEntry) e.nextElement();
      v.addElement( new DistEntry( d.p, new Abstr(x, d.x) ));
    }

    return v;
  }

  /** Convert this to a String. */
  public String toString() {
    return "(\\" + x.toString() + ". " + s.toString() + ")";
  }
}

