
package typo.term;

import java.util.*;

/** An application term. */
public class Appl implements Term {

  /** The term being applied. */
  public Term rator;

  /** The argument. */
  public Term rand;

  /** Constructor. */
  public Appl(Term rator, Term rand) {
    this.rator = rator; this.rand = rand;
  }

  /** Substitute into this term. */
  public Term substitute(Var y, Term s) {

    // XXX: this works, but creates lots of new objects.
    // not worrying about that for now...
    return new Appl(rator.substitute(y, s),
                    rand.substitute(y, s));
  }

  /** The interesting case for headReduceOnce. */
  public Term headReduceOnce() {
    // XXX this isn't lazy hardly at all

    Term rator1 = rator.headReduceOnce();
    Term rand1 = rand.headReduceOnce();

    // is this a redex?
    if (rator1 instanceof Abstr) {
      Abstr a = (Abstr) rator1;
      return a.s.substitute(a.x, rand1);   // yup; reduce it
    }

    // otherwise, just use the rator and rand, reduced once
    return new Appl(rator1, rand1);
  }

  public boolean alphaEquals(Term x) {
    if (x instanceof Appl) {  // if x is also an Appl...
      Appl a = (Appl) x;
      return rator.alphaEquals(a.rator) && rand.alphaEquals(a.rand);
    }       // check if the rator and rand are the same

    return false;    // otherwise, x is different
  }

  public Vector dist() {
    Vector v = new Vector();

    // get the possibilities for the rator and rand
    Vector ratorDist = rator.dist();
    Vector randDist = rand.dist();

    for(Enumeration e1=ratorDist.elements(); e1.hasMoreElements();) {
      DistEntry ratorProb = (DistEntry) e1.nextElement();
      for(Enumeration e2=randDist.elements(); e2.hasMoreElements();) {
        DistEntry randProb = (DistEntry) e2.nextElement();
        Term t;

        // if this is a redex
        if (ratorProb.x instanceof Abstr) {
          Abstr a = (Abstr) ratorProb.x;
          t = a.s.substitute(a.x, randProb.x);   // reduce it
        }
        else {    // otherwise, don't make any change
          t = new Appl(ratorProb.x, randProb.x);
        }

        DistEntry p = new DistEntry(ratorProb.p + randProb.p, t);
        v.addElement( p );
      }
    }
// FIXME: but for now return the un-simplified distribution
//    return Utils.simplifyDistribution( v );
// System.out.println("in Appl.dist(): v = " + v.toString());
    return v;
  }

  /** Print this term as a String. */
  public String toString() {
    return "(" + rator.toString() + " " + rand.toString() + ")";
  }

}

