package typo.term;

import java.util.*;

/** A named constant.  These are simply meant to be distinguishable
 * by "eq". */
public class Const implements Term {

  /** Name of this constant. */
  public String s;

  /** Constructor. */
  public Const(String s) {
    this.s = s;
  }

  /** Substituting into this does nothing... */
  public Term substitute(Var y, Term s) {
    return this;
  }

  /** ... and reducing is easy. */
  public Term headReduceOnce() {
    return this;
  }

  public boolean alphaEquals(Term t) {
    if (t instanceof Const) {   // if t is also a Const...
      Const c = (Const) t;
      return c.s.equals(s);     // see if it has the same name
    }

    return false;     // otherwise, t is different
  }

  public Vector dist() {
    Vector v = new Vector();

    // this will always return itself
    v.addElement( new DistEntry(0, this) );

    return v;
  }

  /** Convert this term to a String. */
  public String toString() {
    return s + " ";
  }
}

