
package typo.term;

import java.util.*;

/** "eq" operator -- reduces both sides to normal form, and
 * returns "true" if they're equal, false otherwise. */
public class Equal implements Term {

  /** The terms which will be compared. */
  public Term a, b;

  /** Constructor. */
  public Equal(Term a, Term b) {
    this.a = a; this.b = b;
  }

  /** Substitute into this term. */
  public Term substitute(Var y, Term s) {

    // XXX: this works, but creates lots of new objects.
    // not worrying about that for now...
    return new Equal(a.substitute(y, s),
                     b.substitute(y, s));
  }

  /** The interesting case for headReduceOnce. */
  public Term headReduceOnce() {
    // FIXME: should make sure a and b are _closed_ terms, and if
    // they aren't, not do a reduction
    if (a instanceof Const && b instanceof Const)
      return toChurchBoolean( a.alphaEquals(b) );
    else
      return this;
  }

  public boolean alphaEquals(Term x) {
    if (x instanceof Equal) {
      Equal e = (Equal) x;
      return a.alphaEquals(e.a) && b.alphaEquals(e.b);
    }

    return false;
  }

  public Vector dist() {
    Vector v = new Vector();

    // FIXME: check for terms being closed (see this.headReduceOnce() )
    for(Enumeration e1=a.dist().elements(); e1.hasMoreElements(); ) {
      DistEntry aDist = (DistEntry) e1.nextElement();
      for(Enumeration e2=b.dist().elements(); e2.hasMoreElements(); ) {
        DistEntry bDist = (DistEntry) e2.nextElement();

	// if they're in NF, compare them, otherwise wait
        if (aDist.x instanceof Const &&
            bDist.x instanceof Const)
          v.addElement(new DistEntry( aDist.p + bDist.p,
	  	 toChurchBoolean( aDist.x.alphaEquals(bDist.x))));
	else
	  v.addElement(new DistEntry( aDist.p + bDist.p,
		new Equal(aDist.x, bDist.x)));
      }
    }
//    return Utils.simplifyDistribution( v );
// System.out.println("in Equal.dist(): v = " + v.toString());
    return v;
  }

  /** Utility mapping True -> K and False -> KI. */
  private Term toChurchBoolean(boolean b) {
    Var t = new Var("t"); Var f = new Var("f");
    Term z = f;
    if (b) z = t;
    return new Abstr(t, new Abstr(f, z));
  }

  /** Print this term as a String. */
  public String toString() {
    return "(" + a.toString() + " = " + b.toString() + ")";
  }

}

