
package typo.term;

import java.util.*;

/** A coin toss. */
public class Flip implements Term {

  /** The probability that this evaluates to True.
   * (Note that this _isn't_ a log probability.) */
  public double p;

  /** Source of randomness.  This may be removed... */
  Random r;

  public Flip(double p) {
    this.p = p;
  }

  /** Constructor. */
  public Flip(double p, Random r) {
    this.p = p;
    this.r = r;
  }

  /** Substituting into this term doesn't do anything. */
  public Term substitute(Var y, Term t) {
    return this;
  }

  public Term headReduceOnce() {
    // FIXME: use a random number generator object?
    if (Math.random() <= p)
      return new Const("True");
    else
      return new Const("False");
  }

  public boolean alphaEquals(Term t) {
    return (t == this);
  }

  public Vector dist() {
    Vector v = new Vector();

    // with probability p, this is True; otherwise it's False
    v.addElement( new DistEntry( Math.log( p ), new Const("True") ));
    v.addElement( new DistEntry( Math.log(1 - p ), new Const("False") ));

    return v;
  }

  /** Convert this to a String. */
  public String toString() {
    return "flip(" + p + ")";
  }
}

