
package typo.term;

import java.io.Serializable;
import java.util.*;

/** An attempt at implementing terms using references to
 * handle variable binding.
 * 20020323, jtb: throwing out types (for now), adding randomness */
public interface Term extends Serializable {

  /** Calculates the result of substituting into this term.
   * If possible, it returns a copy, but it shouldn't
   * destructively modify anything.
   * @param y  the Var to substitute in for
   * @param t  the term to substitute in.
   * @return  the result of substituting */
  public Term substitute(Var y, Term t);

  /** Reduces this term once.  (Preferably in lazy order, but
   * that's not required.)
   * @return  this term, reduced at least once, or unchanged
   *   if the term is already in normal form */
  public Term headReduceOnce();

  /** Compares this term to another term, up to variable
   * renaming.
   * @param t  the term to compare this to
   * @return  true iff this and <code>t</code> are equal
   *   (ignoring the names of variables) */
  public boolean alphaEquals(Term t);

  /** Computes the distribution of terms this might reduce to. */
  public Vector dist();

  /** Convert this term to a string.  There's no guarantee that
   * the resulting term will be parseable. */
  public String toString();

}

