package typo.term;

/** Term utilities.
 * @author Josh T. Burdick */
public final class Utils {

  /** Looks up the type of a Field, traversing the Record types.
   * @param m  the record Term being typechecked
   * @param s  the Record type to search through
   * @param field  the name of the field to find
   * @return  the type of that field */
/*
  Term getFieldType(Term m, Term s, String field)
    throws TypecheckFailedException {
    Term t = s;

    for(;;) {
      t = reduceToRecordOnOutside(t);
// System.out.println("t: " + t.toString());
      if (t instanceof EmptyRecordType)   // if it's an empty record,
        return new EmptyRecordType();    // return an empty record
      if (t instanceof Record) {
        Record r = (Record) t;
        if (field.equals(r.x.name))    // is it this field?
          return r.A;                   // yes; return A
        else       // nope; look in the rest of the Sigma type
          return getFieldType(m, r.B, field)
                   .substitute(r.x, new Field(m, r.x.name));
      }
      else      // it wasn't a record, so throw an exception
        throw new TypecheckFailedException();
    }
  }
*/
  /** Checks if two terms are convertable, modulo
   * alpha-, beta-, and eta- conversion.
   * Assumes they've both been typechecked already, but does
   * check to make sure they're the same type.
   *   For now, it just reduces both to normal form.
   * @param x, y  the terms to compare
   * @return  true iff x and y have the same normal form. */
  public static boolean convertable(Term x, Term y) {
// System.out.print("B");
    Term t1 = normalise(x);
    Term t2 = normalise(y);
// System.out.print("b");
    return t1.alphaEquals(t2);
  }

  /** Reduces a term to normal form.
   * @param x  the term in question
   @ @return  x in normal form */
  public static Term normalise(Term x) {
    Term x1 = x.headReduceOnce();

// System.out.print("N");
    // keep reducing, until it makes no difference
    while (! x.alphaEquals(x1) ) {
      x = x1;
      x1 = x.headReduceOnce();
// System.out.println("reduced to: " + x1.toString() + "\n");
    }
// System.out.print("n");
    return x1;
  }

}
