package typo.term;

import java.util.*;

/** A variable term. */
public class Var implements Term {

  /** The name of the variable. */
  public String name;

  /** Constructor. */
  public Var(String name) {
    this.name = name;
  }

  /** Constructor for variables without a specific name. */
  public Var() {
    this.name = "_";
  }

  /** Substitute into this term.  Note that we compare object
   * references, not the <code>name</code>; the variable name
   * is just there for debugging.
   * @see Object.equals */
  public Term substitute(Var y, Term s) {
    if (equals(y)) {       // is it this variable?
      return s;            // yes; substitute.
    }
    else {
      return this;         // no; nothing changed.
    }
  }

  /** This variable doesn't have a value yet, so don't do anything
   * to reduce it. */
  public Term headReduceOnce() {
    return this;
  }

  public boolean alphaEquals(Term x) {

    // we assume that variables have been substituted so that if the
    // terms are alpha-equal, this will be true
    return this == x;
  }

  public Vector dist() {
    Vector v = new Vector();

    // this will always return itself
    v.addElement( new DistEntry(0, this) );

    return v;
  }

  /** Convert this term to a String. 
   XXX note ugly hack to print unique names. */
  public String toString() {
    int n = this.hashCode() % 10000;
    return name + "_" + n;
  }
}

