; Computes a distribution of the possible values of a function.
; 20020416, jtb: started

(load "list_comb.scm")
(load "utils.scm")




; Computes a distribution of the possible values of a
; stochastic program.
; Args:
;   env - an environment of bindings, as an a-list
;   x - a quoted term representing a stochastic program
; Returns: an a-list of pairs (x . logProb), where
;     x is a term and
;     log_prob is the log probability of that term
(define (dist_eval env x)
;   (begin
;     (write `(dist_eval ,x))
;     (newline)
  
     (cond

      ; things which are in reduced form
      ((reduced? x)
       `((,x . 0)))
              	                  
      ; a variable
      ((symbol? x)
       (dist_eval env (cdr (assoc x env))))

;       ((symbol? x)
;        (merge_distributions
; 	 map
; 	 (lambda (d)
; 	   (let ((x_val (car d))
; 		 (log_prob (cdr d)))
; 	     (weight_distribution
; 	      log_prob
; 	      (dist_eval (blah  ) x
; 	   )
; 	    (dist_eval env (cdr (assoc x env))))))
      
	     
      ; at this point, we assume x is a list...

      ; coin flip
      ((equal? (car x) 'flip)
       (let ((prob (cadr x)))
	 `((#t . ,(log prob)) (#f . ,(log (- 1 prob))))))
      
      ; if statement
      ((eq? (car x) 'if)
       (merge_distributions
	(map (lambda (c)
	       (if (eqv? (car c) #t)
		   (weight_dist (cdr c) (dist_eval env (caddr x)))
		   (weight_dist (cdr c) (dist_eval env (cadddr x)))))
	     (dist_eval env (cadr x)))))

      ; lambda abstraction
;     ((eq? (car x) 'lambda)
;        (let ((vars (cadr x)))
; 	 (let ((body (caddr x)))
; 	   (map (lambda (d)
; 		  (let ((x (car d))
; 			(log_prob (cdr d)))
; 		    `((lambda ,vars ,x) . ,log_prob)))
; 		(dist_eval env body)))))
; ??? not at all sure about this...
      ((eq? (car x) 'lambda)
       `((,x . 0)))

      ; application
      (else
       (let ((head_dist (dist_eval env (car x)))
	     (arg_dist_thunks
	      (map
	       (lambda (a) (dist_eval env a))
	       (cdr x))))
	 (merge_distributions
	  (map
	   (lambda (x) (dist_apply env x))
	   (list_combinations
	    (cons head_dist arg_dist_thunks))))))))

; computes the distribution for an application.  This will be
; a beta-redex, or a (lazy or strict) built-in.
; Args:
;   env - an environment 
;   x - a list of things of the form (log_prob term) 
; Returns: something of the form
;   (log_prob term)
(define (dist_apply env x)
  (let (
	; the log-probability of this is the sum of the log-probability
	; of each subterm reducing to the given value
	(log_prob (apply + (map cdr x)))

	; the redex, as a list of terms
	(a (map car x))

	; the head of the redex
	(h (caar x)))
    (cond

     ((symbol? h)
      `(,a . ,log_prob))    ; ???

     ; if this is a redex, bind values to vars., and recurse
     ((and (pair? h) (eq? (car h) 'lambda))
      (let ((vars (cadr h))
	    (body (caddr h)))
	(let ((new_env (append (zip vars (cdr a)) env)))
	  (weight_dist log_prob (dist_eval new_env body)))))

     ((pair? h)
      (weight_dist log_prob (dist_eval env (reduce a))))
)))

; make two lists into a pair of lists
; Args:
;   a, b - two lists
; Returns: ((a_1 . b_1) (a_2 . b_2) (a_3 . b_3) ...), where a_1
;   is the first element of a, etc.
(define (zip a b)
  (if (null? a)
      '()
      `((,(car a) . ,(car b)) . ,(zip (cdr a) (cdr b)))))

; merge several distributions together
; Args:
;   dists - a distribution, as an a-list
; Returns: a merged distribution, as an a-list
; x is a term
(define (merge_distributions dists)
  (simplify_dist
   (reduce append dists)))   ; ??? not sure if this is right

; defines what is in "reduced form", and doesn't need any more reducing
(define (reduced? x)
  (or (boolean? x)
      (string? x)
      (number? x)))


