; Find which variables we actually need to know the
; distributions of.  (see Koller et al paper)

(require 'common-list-functions)

; Finds "the minimal set of variables in x's cone that
; d-separate the cone from y"
; Args:
;   env - an environment, as a list of (var value) 2-element lists
;   v - a list of variables
;   x - a variable 				       
; Returns: a list of variables
(define (seenby env v x)
  ; remove duplicates (assuming sublists don't have duplicates)
  (reduce
   union
   ; combine everything seen by the variables in v
   (map (lambda (y) (seenby1 env y x)) v)))

; This just finds a minimal set of variables
; "seen by y above x" which d-separates x and y.
; Args:
;   env - an environment
;   y - a variable
;   x - a variable
; Returns: a list of variables
(define (seenby1 env y x)
  '())   ; FIXME


; Given an expression, finds all the variables in a network that it
; depends on, directly or indirectly.
;   Note that this estimates such things pessimistically -- an "if"
; statement is assumed to depend on both of its branches, although
; it only really depends on one branch.
;   (Also note that there are probably much more efficient algorithms
; to do this.)
; Args:
;   x - an expression
;   n - a network
; Returns: the variables in n that x depends on
(define (dependencies x n)
  (let ((add_to_list
	 (lambda (vars)
	   (reduce-init union vars
			(map
			  (lambda (x)
			    (free_vars
			      (cdr (assoc x n))))
			  vars)))))
  (do ((d `(,x) (add_to_list d)))
      ((eqv? (set-difference (add_to_list d) d) '())
       d))
))

; Finds the free variables of an expression (which are the things
; it directly depends on.)
; Args:
;   x - an expression
; Returns: a list of variables
(define (free_vars x)
  (cond

   ((and (symbol? x) (not (memv x builtin_names)))
    `(,x))

   ((list? x)
    (reduce-init union '() (map free_vars x)))

   (else '())
))

; things which shouldn't be included
(define builtin_names '(if flip lambda #t #f))


(free_vars 'a)
(free_vars '(a b))
(free_vars #t)
(free_vars '(if #t a b))

(dependencies 'a '((a . b) (b . 5) (c . #t)))
(dependencies 'b '((a . b) (b . 5)))
(dependencies 'a '((a . (if #t #f b)) (b . 42)))




