; various utilities

(require 'common-list-functions)

; The cutoff for how much smaller a logarithm is before
; we just ignore it.
(define log_sum_cutoff 18)

; Given the logarithms of some numbers, computes the logarithm
; of their sum.
;   Based on the algorithm described in
; Manning & Schutze.  _Foundations of Statistical Natural
;   Language Processing_, p. 337.
; Args:
;   a - a list of numbers (which are the logarithms of the
;       numbers to be added)
; Returns: the log of their sum
(define (log_sum a)
  (let ((m (apply max a)))
    (let ((a1 (remove-if
	       (lambda (x) (< x (- m log_sum_cutoff)))
	       a)))
      (let ((a_scaled_exp (map (lambda (x) (exp (- x m))) a1)))
	(+ m (log (reduce + a_scaled_exp)))))))

; Remove duplicates from a list.
(define (remove_duplicates a)
  (if (null? a)
      '()
      (let ((hd (car a))
	    (tl (cdr a)))
      (if (memv hd tl)
	  (remove_duplicates tl)
	  (cons hd (remove_duplicates tl)))))) 

; simplify a distribution
; this isn't nearly as fast as it could be
(define (simplify_dist x)
  (let ((terms (remove_duplicates (map car x))))
    (map
     (lambda (t)
       (cons
	t
	(log_sum
	 (map cdr
	      (remove-if-not (lambda (a) (eqv? t (car a)))
			     x)))))
     terms)))

; (simplify_dist '((a . 0) (a . 0) (b . 0) (b . 0) (b . 0)))
	     
; weight a distribution by some amount
; Args:
;   w - a log probability
;   dist - a distribution
; Returns: dist, with w added to all the log-probabilities
(define (weight_dist w dist)
  (map
   (lambda (x)
     (cons (car x) (+ w (cdr x))))
   dist))
  
